# frozen_string_literal: true

module AppleSignIn
  class IdentityTokenVerifier
    include AutoInject[
      :apple_base_url,
      :apple_api_caller,
      :apple_client_ids
    ]

    def valid?(identity_token)
      decoded_token = JSON::JWT.decode(identity_token, :skip_verification)

      valid_claims?(decoded_token) && 
        valid_headers?(decoded_token.header) &&
        valid_signature?(identity_token)
    end

    private

    def valid_claims?(claims)
      valid_issuer?(claims) &&
        valid_audience?(claims) &&
        valid_time?(claims) &&
        valid_expiry_time?(claims)
    end

    def valid_issuer?(claims)
      claims["iss"].include?(apple_base_url.to_s)
    end

    def valid_audience?(claims)
      apple_client_ids.include?(claims["aud"])
    end

    def valid_time?(claims)
      claims["iat"].between?(30.seconds.ago.to_i, Time.now.to_i)
    end

    def valid_expiry_time?(claims)
      claims["exp"] > Time.now.to_i
    end

    def valid_headers?(headers)
      headers["alg"] == "RS256"
    end

    def valid_signature?(identity_token)
      jwt = JSON::JWT.decode(identity_token, :skip_verification)
      kid = jwt.header["kid"]
      key = select_public_key(kid)
      jwt.verify!(key)
    end

    def apple_public_keys
      response = apple_api_caller.get("/auth/keys")
      JSON.parse(response.body)["keys"]
    end

    def select_public_key(kid)
      jwk_set = JSON::JWK::Set.new(apple_public_keys)
      appropriate_key = jwk_set.select { |key| key["kid"] == kid }.first
      appropriate_key.to_key
    end
  end
end