# frozen_string_literal: true

module AppleSignIn
  class RefreshTokenRetriever
    include AutoInject[
      :apple_api_caller,
      :apple_client_secret_generator,
      :apple_redirect_uri
    ]

    def call(authorization_data)
      body = build_body(authorization_data)
      response = apple_api_caller.post("/auth/token", body)
      parse(response)
    end

    private

    def parse(response)
      return JSON.parse(response.body) if response.code == 200

      handle_error(response)
    end

    def handle_error(response)
      Rollbar.error(response)
      return raise(AppleSignIn::Error, response.body.to_s) if response.code == 400

      raise(AppleSignIn::Error, response.code.to_s)
    end

    def build_body(authorization_data)
      {
        "client_id" => extract_apple_client_id(authorization_data["identity_token"]),
        "client_secret" => secret(authorization_data["identity_token"]),
        "grant_type" => "authorization_code",
        "code" => authorization_data["authorization_code"],
        "redirect_uri" => apple_redirect_uri
      }
    end

    def extract_apple_client_id(identity_token)
      token_payload = JSON::JWT.decode(identity_token, :skip_verification)
      token_payload["aud"]
    end

    def secret(identity_token)
      apple_client_secret_generator.generate(identity_token)
    end
  end
end